@@ -4,8 +4,9 @@
 **
 ** completion.c
 **
-** Author<s>:   Marc Bevand (aka "After") <bevand_m@epita.fr>
+** Author<s>:   Bevand (aka "After") <bevand_m@epita.fr>
 **              Emmanuel Turquin (aka "Ender") <turqui_e@epita.fr>
+**              Michael Terry <mterry@fastmail.fm>
 ** Last update: 2000-05-04
 ** Description: Do auto-completion stuff
 **
@@ -29,12 +30,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "main.h"
 #include "editor.h"
 #include "struct.h"
 #include "msgbar.h"
 #include "prefs.h"
 #include "completion.h"
-
+#include "tools.h"
 
 extern GtkWidget	*MainNotebook;
 extern GArray		*FileProperties;
@@ -43,10 +45,10 @@ extern t_settings	Settings;
 
 /*
 ** This fcn proceed auto-completion, it is called in interface.c by
-** a ItemFactory method... (standart accelerator: <Alt>space)
+** a ItemFactory method... (standard accelerator: <control>space)
 **
 ** Parameters :
-**  Editor		The GtkText where completion is needed
+**  Editor		The GtkTextView where completion is needed
 **
 ** Return values :
 ## -2                   The Language has no delimiters
@@ -54,8 +56,10 @@ extern t_settings	Settings;
 ** 0			OK
 */
 
-gint			auto_completion(GtkText *Editor)
+gint			auto_completion(GtkTextView *Editor)
 {
+  /* Turned off auto-completion for now. */
+#if 0
   GtkWidget		*CompletionWindow;
   GtkWidget		*scrolled_window;
   GtkWidget		*gtklist;
@@ -72,33 +76,48 @@ gint			auto_completion(GtkText *Editor)
   gchar			*keyword;
   gint			Lg;
   gint			col;
+  gchar			*buffer;
+  GtkTextBuffer		*EditorBuffer;
+  GtkTextIter		start, end;
+  GdkRectangle		rect;
 
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): Begin\n", __func__);
+#endif
   Lg = FPROPS(gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook)),
 	      WidgetInfo.Lg);
   if (Lg == -1)
     {
-      print_msg ("Cannot complete: language not recognized...");
+      print_msg(_("Cannot auto-complete: language not recognized."));
       return -1;
     }
-  ebok = gtk_editable_get_position(GTK_EDITABLE(Editor));
+  EditorBuffer = gtk_text_view_get_buffer (Editor);
+  gtk_text_buffer_get_bounds(EditorBuffer, &start, &end);
+  buffer = gtk_text_buffer_get_text(EditorBuffer, &start, &end, TRUE);
+  gtk_text_buffer_get_iter_at_mark(EditorBuffer, &start,
+                  gtk_text_buffer_get_mark(EditorBuffer, "insert"));
+  ebok = gtk_text_iter_get_offset(&start);
   ibok = ebok - 1;
   if (!Prefs.L[Lg].Delimiters)
     return -2;
   if (Prefs.L[Lg].IsHTML)
     while (ibok > 0 &&
-	   (GTK_TEXT_INDEX(Editor, (guint)ibok) == '<' ||
-	    GTK_TEXT_INDEX(Editor, (guint)ibok) == '/' ||
+	   (buffer[ibok] == '<' ||
+	    buffer[ibok] == '/' ||
 	    !strchr(Prefs.L[Lg].Delimiters,
-		    GTK_TEXT_INDEX(Editor, (guint)ibok))))
+		    buffer[ibok])))
       ibok--;
   else
     while (ibok > 0 &&
 	   !strchr(Prefs.L[Lg].Delimiters,
-		   GTK_TEXT_INDEX(Editor, (guint)ibok)))
+		   buffer[ibok]))
       ibok--;
-  if (strchr(Prefs.L[Lg].Delimiters, GTK_TEXT_INDEX(Editor, (guint)ibok)))
+  if (strchr(Prefs.L[Lg].Delimiters, buffer[ibok]))
     ibok++;
-  bok = gtk_editable_get_chars(GTK_EDITABLE(Editor), (guint)ibok, ebok);
+  g_free(buffer);
+  gtk_text_buffer_get_iter_at_offset(EditorBuffer, &start, ibok);
+  gtk_text_buffer_get_iter_at_offset(EditorBuffer, &end, ebok);
+  bok = gtk_text_buffer_get_text(EditorBuffer, &start, &end, FALSE);
   if (bok)
     {
       for (col = 0; col < MAX_COL; col++)
@@ -116,7 +135,7 @@ gint			auto_completion(GtkText *Editor)
 		    if (!strncmp(bok, keyword, ebok - ibok))
 		      {
 			gtklistitem = gtk_list_item_new_with_label(keyword);
-			gtk_signal_connect (GTK_OBJECT(gtklistitem),
+			g_signal_connect (G_OBJECT(gtklistitem),
 					    "button_press_event",
 					    GTK_SIGNAL_FUNC
 					    (auto_completion_double_clic),
@@ -129,7 +148,7 @@ gint			auto_completion(GtkText *Editor)
 		    if (!strncasecmp(bok, keyword, ebok - ibok))
 		      {
 			gtklistitem = gtk_list_item_new_with_label(keyword);
-			gtk_signal_connect (GTK_OBJECT(gtklistitem),
+			g_signal_connect (G_OBJECT(gtklistitem),
 					    "button_press_event",
 					    GTK_SIGNAL_FUNC
 					    (auto_completion_double_clic),
@@ -146,23 +165,24 @@ gint			auto_completion(GtkText *Editor)
     }
   if (!bok || (dlist == NULL))
     {
-      print_msg ("No matching string...");
+      print_msg(_("No matching string."));
       return 0;
     }
   if (!dlist->next)
     {
-      gint		current_pos;
       gchar		*label, *msg;
 
       /*
       ** Only 1 keyword, insert it directly
       */
-      current_pos = gtk_editable_get_position(GTK_EDITABLE(Editor));
+      gtk_text_buffer_get_iter_at_mark(EditorBuffer, &end,
+                  gtk_text_buffer_get_mark(EditorBuffer, "insert"));
+      gtk_text_buffer_get_iter_at_offset(EditorBuffer, &start, ibok);
       gtk_label_get(GTK_LABEL(GTK_BIN(dlist->data)->child), &label);
-      gtk_editable_delete_text(GTK_EDITABLE(Editor), ibok, current_pos);
-      gtk_editable_insert_text(GTK_EDITABLE(Editor),
-			       label, strlen(label), &current_pos);
-      msg = g_strconcat ("String \"", label, "\" inserted...", NULL);
+      gtk_text_buffer_delete(EditorBuffer, &start, &end);
+      gtk_text_buffer_insert_at_cursor(EditorBuffer,
+			       label, strlen(label));
+      msg = g_strdup_printf(_("String \"%s\" inserted."), label);
       print_msg (msg);
       g_free (msg);
       return 0;
@@ -171,6 +191,11 @@ gint			auto_completion(GtkText *Editor)
   ** GtkList contains 2 or more elements. So, create a window, which will
   ** contain auto-completion words
   */
+  gtk_text_buffer_get_iter_at_mark(EditorBuffer, &end,
+                  gtk_text_buffer_get_mark(EditorBuffer, "insert"));
+  gtk_text_view_get_iter_location(Editor, &end, &rect);
+  gtk_text_view_buffer_to_window_coords (Editor, GTK_TEXT_WINDOW_WIDGET,
+	rect.x, rect.y, &rect.x, &rect.y);
   CompletionWindow = gtk_window_new(GTK_WINDOW_POPUP);
   gtk_window_set_title (GTK_WINDOW(CompletionWindow), "Complete");
   gtk_window_set_policy (GTK_WINDOW(CompletionWindow), FALSE, FALSE, FALSE);
@@ -178,22 +203,22 @@ gint			auto_completion(GtkText *Editor)
   gtk_window_set_transient_for
     (GTK_WINDOW(CompletionWindow),
      GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(Editor))));
-  gtk_widget_set_usize(CompletionWindow,
+  gtk_window_set_default_size (GTK_WINDOW (CompletionWindow),
 		       COMPLETE_WINDOW_WIDTH, COMPLETE_WINDOW_HEIGHT);
   beaver_gdkwin = gtk_widget_get_parent_window(GTK_WIDGET(Editor));
   gdk_window_get_origin(beaver_gdkwin, &beaver_x, &beaver_y);
   coord = g_malloc (2*sizeof(gint));
   coord[0] =
     beaver_x + GTK_WIDGET(GTK_WIDGET(MainNotebook)->parent)->allocation.x +
-    GTK_WIDGET(Editor)->allocation.x + Editor->cursor_pos_x + 3;
+    GTK_WIDGET(Editor)->allocation.x + rect.x + 3;
   coord[1] =
     beaver_y + GTK_WIDGET(GTK_WIDGET(MainNotebook)->parent)->allocation.y +
-    GTK_WIDGET(Editor)->allocation.y + Editor->cursor_pos_y + 4;
+    GTK_WIDGET(Editor)->allocation.y + rect.y + 4;
   if (coord[0] > gdk_screen_width() - COMPLETE_WINDOW_WIDTH)
     coord[0] -= COMPLETE_WINDOW_WIDTH;
   if (coord[1] > gdk_screen_height() - COMPLETE_WINDOW_HEIGHT)
     coord[1] -= COMPLETE_WINDOW_HEIGHT + 15;
-  gtk_widget_set_uposition(CompletionWindow, coord[0], coord[1]);
+  gtk_window_move (GTK_WINDOW (CompletionWindow), coord[0], coord[1]);
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scrolled_window),
 				  GTK_POLICY_AUTOMATIC,
@@ -214,24 +239,25 @@ gint			auto_completion(GtkText *Editor)
   gtk_list_append_items(GTK_LIST(gtklist), dlist);
   gtk_window_set_focus(GTK_WINDOW(CompletionWindow),
 		       GTK_WIDGET(GTK_LIST(gtklist)->children->data));
-  gtk_widget_show_all(CompletionWindow);
-  /*
-  ** Now, the window is displayed and we need to connect signals and
-  ** do other stuff...
-  */
   datas.ebok = ebok;
   datas.ibok = ibok;
-  gtk_signal_connect(GTK_OBJECT(CompletionWindow), "key_press_event",
+  g_signal_connect(G_OBJECT(CompletionWindow), "key_press_event",
 		     GTK_SIGNAL_FUNC(auto_completion_key_press), &datas);
-  gtk_signal_connect(GTK_OBJECT(CompletionWindow), "button_press_event",
+  g_signal_connect(G_OBJECT(CompletionWindow), "button_press_event",
 		     GTK_SIGNAL_FUNC(stop_completion), coord);
-  gtk_signal_connect(GTK_OBJECT(CompletionWindow), "delete_event",
+  g_signal_connect(G_OBJECT(CompletionWindow), "delete_event",
 		     GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);
-  gtk_signal_connect(GTK_OBJECT(CompletionWindow), "destroy_event",
+  g_signal_connect(G_OBJECT(CompletionWindow), "destroy_event",
 		     GTK_SIGNAL_FUNC(gtk_widget_destroy), NULL);
+  gtk_widget_show_all(CompletionWindow);
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
+#endif
   return 0;
 }
 
+#if 0
 /*
 ** This fcn is called when a a mouse button is pressed out of the completion
 ** window: it destroys the window...
@@ -246,11 +272,17 @@ void			stop_completion(GtkWidget *widget,
 					GdkEventButton *ev,
 					gint *coord)
 {
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): Begin\n", __func__);
+#endif
   if ((ev -> x_root < coord[0]) ||
       (ev -> y_root < coord[1]) ||
       (ev -> x_root > coord[0]+COMPLETE_WINDOW_WIDTH) ||
       (ev -> y_root > coord[1]+COMPLETE_WINDOW_HEIGHT))
     gtk_widget_destroy (gtk_widget_get_toplevel(widget));
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
 }
 
 /*
@@ -267,26 +299,37 @@ void			auto_completion_double_clic(GtkWidget *widget,
 						    GdkEventButton *ev,
 						    t_auco_datas *datas)
 {
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): Begin\n", __func__);
+#endif
   if (ev -> type == GDK_2BUTTON_PRESS)
     {
       gchar		*label, *msg;
       gint			CurrentPage, current_pos;
-
+      GtkTextBuffer *Buffer;
+      
       CurrentPage = gtk_notebook_get_current_page
 	(GTK_NOTEBOOK(MainNotebook));
-      current_pos = gtk_editable_get_position
-	(GTK_EDITABLE(FPROPS(CurrentPage, Text)));
+      Buffer = FPROPS(CurrentPage, Buffer);
+      current_pos = get_position (Buffer);
       gtk_label_get (GTK_LABEL(GTK_BIN(widget)->child), &label);
       if (current_pos > datas->ibok)
-	gtk_editable_delete_text (GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-				  datas->ibok, current_pos);
-      gtk_editable_insert_text(GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-			       label, strlen (label), &current_pos);
-      msg = g_strconcat ("String \"", label, "\" inserted...", NULL);
+      {
+      	GtkTextIter start, end;
+      	
+      	gtk_text_buffer_get_iter_at_offset (Buffer, &start, datas->ibok);
+      	gtk_text_buffer_get_iter_at_offset (Buffer, &end, current_pos);
+	gtk_text_buffer_delete (Buffer, &start, &end);
+      }
+      gtk_text_buffer_insert_at_cursor(Buffer, label, -1);
+      msg = g_strdup_printf(_("String \"%s\" inserted."), label);
       print_msg (msg);
       g_free (msg);
       gtk_widget_destroy(gtk_widget_get_toplevel(widget));
     }
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
 }
 
 /*
@@ -303,11 +346,15 @@ void			auto_completion_key_press(GtkWidget *window,
 						  t_auco_datas *datas)
 {
   gint			CurrentPage, current_pos;
+  GtkTextBuffer *Buffer;
   
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): Begin\n", __func__);
+#endif
   CurrentPage = gtk_notebook_get_current_page
     (GTK_NOTEBOOK(MainNotebook));
-  current_pos = gtk_editable_get_position
-    (GTK_EDITABLE(FPROPS(CurrentPage, Text)));
+  Buffer = FPROPS(CurrentPage, Buffer);
+  current_pos = get_position (FPROPS(CurrentPage, Buffer));
   if (event->string[0] == '\t' ||
       event->string[0] == '\r' ||
       event->string[0] == ' ')
@@ -327,11 +374,15 @@ void			auto_completion_key_press(GtkWidget *window,
 		       ->selection->data)
 		      ->child), &label);
       if (current_pos > datas->ibok)
-	gtk_editable_delete_text (GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-				  datas->ibok, current_pos);
-      gtk_editable_insert_text(GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-			       label, strlen (label), &current_pos);
-      msg = g_strconcat ("String \"", label, "\" inserted...", NULL);
+      {
+      	GtkTextIter start, end;
+      	
+      	gtk_text_buffer_get_iter_at_offset (Buffer, &start, datas->ibok);
+      	gtk_text_buffer_get_iter_at_offset (Buffer, &end, current_pos);
+	gtk_text_buffer_delete (Buffer, &start, &end);
+      }
+      gtk_text_buffer_insert_at_cursor(Buffer, label, -1);
+      msg = g_strdup_printf(_("String \"%s\" inserted."), label);
       print_msg (msg);
       g_free (msg);
       gtk_widget_destroy(window);
@@ -341,13 +392,16 @@ void			auto_completion_key_press(GtkWidget *window,
       if (current_pos > datas->ebok)
 	{
 	  gchar *word;
+	  GtkTextIter current, back;
+	  GtkTextBuffer *Buffer;
 	  
-	  gtk_editable_delete_text (GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-				    current_pos-1, current_pos);
-	  word = gtk_editable_get_chars
-	    (GTK_EDITABLE(FPROPS(CurrentPage, Text)), datas->ibok,
-	     gtk_editable_get_position
-	     (GTK_EDITABLE(FPROPS(CurrentPage, Text))));
+	  Buffer = FPROPS(CurrentPage, Buffer);
+	  gtk_text_buffer_get_iter_at_offset (Buffer, &current,
+	     get_position (Buffer));
+	  back = current;
+	  gtk_text_iter_backward_char (&back);
+	  gtk_text_buffer_delete (Buffer, &back, &current);
+	  word = get_chars (Buffer, datas->ibok, get_position (Buffer));
 	  refresh_selection (GTK_LIST
 			     (GTK_BIN
 			      (GTK_BIN
@@ -361,7 +415,7 @@ void			auto_completion_key_press(GtkWidget *window,
       else
 	{
 	  if (BEEP) gdk_beep();
-	  print_msg ("Beginning of auto-completion buffer...");
+	  print_msg (_("Beginning of auto-completion buffer."));
 	}
     }
   else if (event->string[0] > ' ' && event->string[0] <= '~')
@@ -369,7 +423,7 @@ void			auto_completion_key_press(GtkWidget *window,
       gchar *word;
       gint len;
 
-      word = gtk_editable_get_chars (GTK_EDITABLE(FPROPS(CurrentPage, Text)),
+      word = get_chars (FPROPS(CurrentPage, Buffer),
 				     datas->ibok, current_pos);
       len = strlen(word);
       word = g_realloc (word, len+2);
@@ -383,12 +437,12 @@ void			auto_completion_key_press(GtkWidget *window,
 				->child)
 			       ->child)
 			      ->child), word))
-	gtk_editable_insert_text(GTK_EDITABLE(FPROPS(CurrentPage, Text)),
-				 event->string, 1, &current_pos);
+	gtk_text_buffer_insert_at_cursor(FPROPS(CurrentPage, Buffer),
+				 event->string, 1);
       else
 	{
 	  if (BEEP) gdk_beep();
-	  print_msg ("No matching string...");
+	  print_msg(_("No matching string."));
 	}
       g_free (word);
     }
@@ -413,6 +467,9 @@ void			auto_completion_key_press(GtkWidget *window,
 	   event->keyval != 0xff9c &&   /* KP_End */
 	   event->keyval != 0xffe5)     /* Caps Lock */
     gtk_widget_destroy(window);
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
 }
 
 /*
@@ -436,6 +493,9 @@ gboolean		refresh_selection(GtkList *gtklist, gchar *word)
   gchar			*label;
   gint			Lg;
 
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): Begin\n", __func__);
+#endif
   Lg = FPROPS(gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook)),
 	      WidgetInfo.Lg);
   item = gtklist->children;
@@ -463,8 +523,17 @@ gboolean		refresh_selection(GtkList *gtklist, gchar *word)
 	  position /= len - 1;
 	  gtk_list_scroll_vertical(gtklist, GTK_SCROLL_JUMP, position);
 	}
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
       return TRUE;
     }
   else
+  {
+#ifdef DEBUG_FCN
+  g_print(__FILE__": %s(): End\n", __func__);
+#endif
     return FALSE;
+  }
 }
+#endif
