@@ -73,7 +73,7 @@ gint		get_section_limits(gchar *section, t_conf *conf)
   section_eq = g_strconcat("[" , section, "]", NULL);
   len = strlen(section_eq);
   for (i = 0; conf->file_content[i]; i++)
-    if (!g_strncasecmp(conf->file_content[i], section_eq, len))
+    if (!g_ascii_strncasecmp(conf->file_content[i], section_eq, len))
       {
 	conf->section[0] = ++i;
 	while (conf->file_content[i] && conf->file_content[i][0] != '[')
@@ -97,7 +97,7 @@ gint		get_key(gchar *key, t_conf *conf)
   key_eq = g_strconcat(key, " = ", NULL);
   len = strlen(key_eq);
   for (i = conf->section[0]; i < conf->section[1]; i++)
-    if (!g_strncasecmp(conf->file_content[i], key_eq, len))
+    if (!g_ascii_strncasecmp(conf->file_content[i], key_eq, len))
       {
 	conf->line = g_realloc(conf->line,
 			       strlen(conf->file_content[i]) - len + 1);
@@ -126,8 +126,7 @@ gint		get_conf(gchar *key, t_conf *conf)
   gint		i, j;
   
   key_tab = g_strsplit(key, DELIM_STR, 0);
-  file_name = g_strconcat(g_get_home_dir(), PATH_SEP_STRING, CONF_DIR,
-			  PATH_SEP_STRING, key_tab[0], NULL);
+  file_name = g_build_filename(g_get_home_dir(), CONF_DIR, key_tab[0], NULL);
   if ((fd = open(file_name, O_RDONLY)) == -1)
     {
       g_free(file_name);
@@ -194,6 +193,21 @@ gint		get_int_conf(gchar *key)
   return (ret_val);
 }
 
+gint		get_int_conf2(gchar *key, gint default_val)
+{
+  t_conf	conf;
+  gint		ret_val;
+
+  if (get_conf(key, &conf))
+    {
+      free_conf(&conf);
+      return default_val;
+    }
+  ret_val = (gint) g_strtod(conf.line, NULL);
+  free_conf(&conf);
+  return (ret_val);
+}
+
 gboolean	get_bool_conf(gchar *key)
 {
   t_conf	conf;
@@ -204,7 +218,7 @@ gboolean	get_bool_conf(gchar *key)
       free_conf(&conf);
       return (0);
     }
-  if (!g_strcasecmp(conf.line, TRUE_STR))
+  if (!g_ascii_strcasecmp(conf.line, TRUE_STR))
     ret_val = 1;
   else
     ret_val = 0;
@@ -212,6 +226,24 @@ gboolean	get_bool_conf(gchar *key)
   return (ret_val);
 }
 
+gboolean	get_bool_conf2(gchar *key, gboolean default_val)
+{
+  t_conf	conf;
+  gboolean	ret_val;
+
+  if (get_conf(key, &conf))
+    {
+      free_conf(&conf);
+      return default_val;
+    }
+  if (!g_ascii_strcasecmp(conf.line, TRUE_STR))
+    ret_val = TRUE;
+  else
+    ret_val = FALSE;
+  free_conf(&conf);
+  return (ret_val);
+}
+
 gchar		*get_string_conf(gchar *key)
 {
   t_conf	conf;
@@ -229,6 +261,21 @@ gchar		*get_string_conf(gchar *key)
   return (ret_val);
 }
 
+gchar		*get_string_conf2(gchar *key, gchar *default_val)
+{
+  t_conf	conf;
+  gchar		*ret_val;
+
+  if (get_conf(key, &conf))
+    {
+      free_conf(&conf);
+      return g_strdup (default_val);
+    }
+  ret_val = parse_string(conf.line);
+  free_conf(&conf);
+  return (ret_val);
+}
+
 gint		find_key(gchar *key, t_conf *conf)
 {
   gint		i;
@@ -240,7 +287,7 @@ gint		find_key(gchar *key, t_conf *conf)
   key_eq = g_strconcat(key, " = ", NULL);
   len = strlen(key_eq);
   for (i = conf->section[0]; i < conf->section[1]; i++)
-    if (!g_strncasecmp(conf->file_content[i], key_eq, len))
+    if (!g_ascii_strncasecmp(conf->file_content[i], key_eq, len))
       {
 	conf->section[0] = i;
 	ret_val = 0;
@@ -261,8 +308,7 @@ gint		set_conf(gchar *key, t_conf *conf)
   gint		i, j;
   
   key_tab = g_strsplit(key, DELIM_STR, 0);
-  file_name = g_strconcat(g_get_home_dir(), PATH_SEP_STRING, CONF_DIR,
-			  PATH_SEP_STRING, key_tab[0], NULL);
+  file_name = g_build_filename(g_get_home_dir(), CONF_DIR, key_tab[0], NULL);
   if ((fd = open(file_name, O_RDONLY)) == -1)
     {
       g_free(file_name);
@@ -291,7 +337,8 @@ gint		set_conf(gchar *key, t_conf *conf)
   line_to_add = g_strconcat(key_tab[2], " = ", conf->line, "\n", NULL);
   if (get_section_limits(key_tab[1], conf))
     {
-      if ((fd = open(file_name, O_APPEND)) == -1)
+      if ((fd = open(file_name, O_CREAT | O_WRONLY | O_APPEND, 
+		S_IREAD | S_IWRITE)) == -1)
 	{
 	  g_free(file_name);
 	  g_free(line_to_add);
