@@ -42,4 +42,8 @@ gint		get_int_conf(gchar *key);
 gboolean	get_bool_conf(gchar *key);
 gchar		*get_string_conf(gchar *key);
 
+gint		get_int_conf2(gchar *key, gint default_val);
+gboolean	get_bool_conf2(gchar *key, gboolean default_val);
+gchar		*get_string_conf2(gchar *key, gchar *default_val);
+
 #endif
