@@ -5,6 +5,7 @@
 ** interface.h
 **
 ** Author<s>:   Emmanuel Turquin (aka "Ender") <turqui_e@epita.fr>
+**              Michael Terry <mterry@fastmail.fm>
 ** Latest update: Thu Nov  9 00:35:08 2000
 ** Description:   Beaver interface source header
 **
@@ -29,11 +30,13 @@
 /* Menu entries for the main menubar and the toolbar */
 
 enum {
-  NEW,
+  NEW = 0,
   OPEN,
   SAVE,
   SAVE_ALL,
   SAVE_AS,
+  SAVE_A_COPY,
+  REVERT,
   PRINT,
   CLOSE,
   CLOSE_ALL,
@@ -47,7 +50,9 @@ enum {
   SELECT_ALL,
   COMPLETE,
   FIND,
+  FIND_IN_FILES,
   REPLACE,
+  REPLACE_IN_FILES,
   LINE,
   READONLY,
   CONVERTER,
@@ -57,6 +62,12 @@ enum {
   TO_LOWER,
   CAPITALIZE,
   INVERT_CASE,
+  TO_UNIX,
+  TO_DOS,
+  TO_MAC,
+  TO_UNIX_ALL,
+  TO_DOS_ALL,
+  TO_MAC_ALL,
   DOS_UNIX,
   UNIX_DOS,
   DOS_MAC,
@@ -79,29 +90,51 @@ enum {
   TAB_POS_RIGHT,
   SCROLLBAR_POS_LEFT,
   SCROLLBAR_POS_RIGHT,
+  TAB_WIDTH_1,
+  TAB_WIDTH_2,
+  TAB_WIDTH_3,
+  TAB_WIDTH_4,
+  TAB_WIDTH_8,
   PREFS,
   LANG,
   HELP,
-  ABOUT
+  ABOUT,
+  SHOW_LINE_NUMBERS,
+  SHOW_MARGIN,
+  NUM_MENU_ACTIONS,
+  RECENT_FILES_OFFSET
 };
 
+void show_error (GtkWindow *parent, const gchar *primary, const gchar *secondary);
+GtkWidget *alert_new (GtkWindow *parent, const gchar *stock, 
+	const gchar *primary, const gchar *secondary);
 void menu_items_treatment (GtkWidget *Widget,
-			   guint Op);
+			   gpointer Op_in);
 GtkWidget *menubar_new (GtkWidget *Window);
 GtkWidget *popup_menu_new (void);
-gboolean popup_menu_show (GtkMenu *PopUpMenu,
-			  GdkEvent *Event);
+gboolean popup_menu_show (GtkMenu *PopUpMenu, GdkEventButton *Event);
 void add_page_in_notebook (GtkNotebook *Notebook,
-			   gchar *Filename);
+			   const gchar *Filename);
 void  set_label (GtkNotebook *Notebook,
 		 gint CurrentPage);
-void set_title (GtkNotebook *Notebook,
-		GtkNotebookPage *Page,
-		gint CurrentPage);
-void command_line (gint argc,
+void set_title (gint CurrentPage);
+gint command_line (gint argc,
 		   gchar *argv[]);
 void interface (gint argc,
 		gchar *argv[]);
 void about (void);
 
+void note_format (void);
+void note_redoundo (gint page);
+void note_saveable (void);
+void menu_manage_documents (void);
+void switch_to_page (gint page);
+void menu_set_sensitive (const gchar *id, gboolean setting);
+
+void set_tab_width (gint width);
+void menu_set_files_open (gboolean opened);
+
+extern GtkWidget *MainWindow;
+extern GtkItemFactory *MainFactory;
+
 #endif
