@@ -5,6 +5,7 @@
 ** languages.h
 **
 ** Author<s>:   Emmanuel Turquin (aka "Ender") <turqui_e@epita.fr>
+**              Michael Terry <mterry@fastmail.fm>
 ** Latest update: Wed May  3 04:23:15 2000
 ** Description:   Beaver languages menus generator source header
 **
@@ -26,8 +27,9 @@
 #ifndef __LANGUAGES_H__
 #define __LANGUAGES_H__
 
+#include "gtksourceview/gtksourcelanguage.h"
+
 void init_languages_menu (void);
-void languages_treatment (GtkWidget *Widget, gint Op);
-void entries_treatment (GtkWidget *Widget, gint Op);
+void languages_activate_menu_item_for_language (GtkSourceLanguage *lang);
 
 #endif
