@@ -5,6 +5,7 @@
 ** msgbar.c
 **
 ** Author<s>:     Emmanuel Turquin (aka "Ender") <turqui_e@epita.fr>
+**                Michael Terry <mterry@fastmail.fm>
 ** Latest update: Wed May  3 04:27:37 2000
 ** Description:   Beaver message bar manipulation source
 **
@@ -25,6 +26,7 @@
 
 #include <gtk/gtk.h>
 #include <sys/stat.h>
+#include "main.h"
 #include "conf.h"
 #include "editor.h"
 #include "struct.h"
@@ -60,7 +62,9 @@ void init_msgbar (GtkBox *ParentBox)
   gtk_box_pack_start (GTK_BOX(MsgBox), MsgLabel, FALSE, FALSE, 0);
   LCLabel = gtk_label_new ("");
   gtk_box_pack_end (GTK_BOX(MsgBox), LCLabel, FALSE, FALSE, 5);
-  gtk_box_pack_end (ParentBox, MsgBar, FALSE, FALSE, 0);
+  gtk_box_pack_end (ParentBox, MsgBar, FALSE, FALSE, 0); 
+  g_signal_connect (G_OBJECT (MsgBar), "delete-event", G_CALLBACK (gtk_widget_destroyed), &MsgBar);
+  g_timeout_add (80, (GSourceFunc)display_line_column, NULL);
 }
 
 
@@ -101,42 +105,50 @@ void print_msg (gchar *Message)
 gint get_line (void)
 {
   GtkWidget *Editor;
-  gint line;
-  guint i;
+  GtkTextIter current;
+  GtkTextBuffer *EditorBuffer;
 
   Editor = FPROPS(gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook)),
 		  Text);
-  line = 1;
-  for (i = 0; i < GTK_TEXT(Editor)->editable.current_pos; i++)
-    if (GTK_TEXT_INDEX(GTK_TEXT(Editor), i) == '\n')
-      line++;
-  return line;
+  EditorBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Editor));
+  gtk_text_buffer_get_iter_at_mark (EditorBuffer, &current, 
+                  gtk_text_buffer_get_insert(EditorBuffer));
+  
+  return gtk_text_iter_get_line(&current) + 1;
 }
 
 
 gint get_column (void)
 {
   GtkWidget *Editor;
-  gint i;
+  GtkTextIter current;
+  GtkTextBuffer *EditorBuffer;
 
   Editor = FPROPS(gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook)),
 		  Text);
-  i = GTK_TEXT(Editor)->editable.current_pos - 1;
-  while ((i >= 0) && (GTK_TEXT_INDEX(GTK_TEXT(Editor), (guint)i) != '\n'))
-    i--;
-  return (gint)GTK_TEXT(Editor)->editable.current_pos - i;
+  EditorBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Editor));
+  gtk_text_buffer_get_iter_at_mark (EditorBuffer, &current, 
+                  gtk_text_buffer_get_insert(EditorBuffer));
+  
+  return gtk_text_iter_get_line_offset(&current) + 1;
 }
 
 
 gint get_percent (void)
 {
   GtkWidget *Editor;
+  GtkTextIter current;
+  GtkTextBuffer *EditorBuffer;
   gint i, j;
 
   Editor = FPROPS(gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook)),
 		  Text);
-  i = 100 * GTK_TEXT(Editor)->editable.current_pos;
-  if ((j = gtk_text_get_length(GTK_TEXT(Editor))) == 0)
+  EditorBuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(Editor));
+  gtk_text_buffer_get_iter_at_mark (EditorBuffer, &current, 
+                  gtk_text_buffer_get_insert(EditorBuffer));
+  
+  i = 100 * gtk_text_iter_get_offset (&current);
+  if ((j = get_num_characters(GTK_TEXT_VIEW(Editor))) == 0)
     return 0;
   else
     return i/j;
@@ -146,18 +158,23 @@ gint get_percent (void)
 gint get_selection_size (void)
 {
   gint CurrentPage;
-
+  gint rv;
+	
   CurrentPage = gtk_notebook_get_current_page(GTK_NOTEBOOK(MainNotebook));
-  return max_from_selection (GTK_EDITABLE(FPROPS(CurrentPage, Text))) -
-    min_from_selection (GTK_EDITABLE(FPROPS(CurrentPage, Text)));
+  rv = max_from_selection (GTK_TEXT_VIEW(FPROPS(CurrentPage, Text))) -
+    min_from_selection (GTK_TEXT_VIEW(FPROPS(CurrentPage, Text)));
+  
+  return rv;
 }
 
 
-gboolean display_line_column (void)
+gboolean display_line_column (gboolean data)
 {
   gchar *line_column;
   gint sel_size;
-  
+
+  if (!MsgBar)
+    return FALSE;
   if (!MSGBAR_DISPLAY) return TRUE;
   if (!OpenedFilesCnt)
     {
@@ -167,11 +184,11 @@ gboolean display_line_column (void)
   while (gtk_events_pending ())
     gtk_main_iteration ();
   if ((sel_size = get_selection_size ()))
-    line_column = g_strdup_printf ("S %03d   L %03d   C %03d   %02d%%",
+    line_column = g_strdup_printf (_("Selected %d   Line %d   Column %d   %d%%"),
 				   sel_size, get_line (), get_column (),
 				   get_percent ());
   else
-    line_column = g_strdup_printf ("L %03d   C %03d   %02d%%",
+    line_column = g_strdup_printf (_("Line %d   Column %d   %d%%"),
 				   get_line (), get_column (), get_percent ());
   gtk_label_set_text (GTK_LABEL(LCLabel), line_column);
   g_free (line_column);
