@@ -26,28 +26,42 @@
 #ifndef __TOOLS_H__
 #define __TOOLS_H__
 
-gint min_from_selection (GtkEditable *Text);
-gint max_from_selection (GtkEditable *Text);
-gchar *get_selection (GtkEditable *Text);
+gint get_num_characters (GtkTextView *Text);
+gint min_from_selection (GtkTextView *Text);
+gint max_from_selection (GtkTextView *Text);
+gchar *get_selection (GtkTextBuffer *Buffer);
+gchar *get_text (GtkTextBuffer *Buffer);
+gint get_position (GtkTextBuffer *Buffer);
+GtkTextIter get_cursor_iter (GtkTextBuffer *Buffer);
+gchar *get_chars (GtkTextBuffer *Buffer, gint left, gint right);
+void replace_selection (GtkTextBuffer *Buffer, const gchar *str);
+void replace_text (GtkTextBuffer *Buffer, GtkTextIter *start, GtkTextIter *end, const gchar *text);
 /* UpDown: 1 = upper case, 2 = lower case, 3 = invert case, 4 = capitalize */
 void change_case (gint UpDown);
 void insert_time (gint CurrentPage);
 void converter_window_not_visible (void);
-void refresh_converter (GtkWidget *Widget, gint EntryID);
+void refresh_converter (GtkWidget *Widget, gpointer data);
 void converter (void);
 void color_picker_not_visible (void);
 void insert_color (GtkColorSelection *csd);
 void color_picker (void);
-gint remove_char (char *string, char to_remove);
-gint replace_char (char *string, char to_replace, char new_char);
 void convert_all(void (*conv_func)(void));
+void convert_this_to_mac (void);
+void convert_this_to_dos (void);
+void convert_this_to_unix (void);
 void conv_unix_to_dos (void);
 void conv_dos_to_unix (void);
 void conv_dos_to_mac (void);
 void conv_mac_to_unix (void);
 void conv_unix_to_mac (void);
 void conv_mac_to_dos (void);
+gint get_format_type (gint CurrentPage);
 void file_info (gint CurrentPage);
 void print_buffer (void);
+gchar *run_program_with_args (const gchar *prog, gchar *const args[], const gchar *input);
+gchar *determine_mime_type (gchar *filename);
+gchar *get_text_from_cursor (GtkTextBuffer *Buffer);
+gchar *get_text_to_cursor (GtkTextBuffer *Buffer);
+gint get_line_of_cursor (GtkTextBuffer *Buffer);
 
 #endif
